/*
Test avcodec seek problem when using ffmpeg-mt's H.264 decoder
Set SEEK_POSITION to an arbitraty timestamp value (23.976 fps MP4/MOV
files typically have a duration of 1001 per frame)
*/

#define SEEK_POSITION 1001 //seek to frame 2 (which is the frame right after the first keyframe)

#include <stdio.h>
#include <sys/types.h>

//avcodec stuff
#include <libavutil/avutil.h>
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>


int main(int argc,char **argv){

	//avcodec read stuff
	AVFormatContext	*formatCtx;
	AVCodecContext	*codecCtx;
	AVCodec			*codec;
	int				videoStream; //keep track of input video stream
	AVFrame			*decoded_frame = NULL;
	AVPacket		packet;
	int				got_picture;
	int64_t			seekpos = SEEK_POSITION;
	int				is_keyframe = 0;
	long			num_frames;
	long			frame_count = 0;
	double			frame_rate;
    int				i;
	
	//init
	av_log_set_level(AV_LOG_QUIET); //comment out to see h.264 decoding errors
	av_register_all();
	
	//open file
	if(!argv[1]){
		fprintf(stderr,"No input file specified\n");
		return -1;
	}
	if(av_open_input_file(&formatCtx, argv[1], NULL, 0, NULL) != 0){
		fprintf(stderr,"Couldn't open input file\n");
		return -1;
    }
	
	//get stream info
	if(av_find_stream_info(formatCtx)<0) return -1; // Couldn't find stream information
	
    //find first video stream
    videoStream = -1;
    for(i=0; i<formatCtx->nb_streams; i++){
        if(formatCtx->streams[i]->codec->codec_type == CODEC_TYPE_VIDEO){
            videoStream=i;
            break;
        }
    }
    if(videoStream == -1) return -1; //couldn't find video stream
    
	
	//get pointer to codec context
	codecCtx = formatCtx->streams[videoStream]->codec;
	
	//set number of threads
	codecCtx->thread_count = 1;
	
	//find a decoder
	codec = avcodec_find_decoder(codecCtx->codec_id);
	if(!codec) return -1; //couldn't find a decoder
	
    //open codec
    if(avcodec_open(codecCtx, codec) < 0) return -1; //couldn't open codec
	
	//retrieve num of frames
	num_frames = formatCtx->streams[videoStream]->nb_frames;
	
	//retrieve frame rate
  	frame_rate = av_q2d(formatCtx->streams[videoStream]->r_frame_rate);
	
	//report findings
	fprintf(stderr,"Frame count of file: %ld frame rate: %.3f\n",num_frames,frame_rate);
		
	//perform seek
	av_seek_frame(formatCtx, videoStream, seekpos, AVSEEK_FLAG_ANY|AVSEEK_FLAG_BACKWARD);
	
	//alloc AVFrame
	decoded_frame = avcodec_alloc_frame();
	
	for(;;){ //decode loop (runs only for one frame/timestamp value when seeking)
	
		//read frame
		av_read_frame(formatCtx, &packet);
		
		if(packet.stream_index == videoStream){ //if from selected video stream
		
			//fprintf(stderr,"was from video stream\n");

			//determine whether this is a keyframe or not
			if(packet.flags & AV_PKT_FLAG_KEY) is_keyframe = 1;
			
			fprintf(stderr,"seekpos:%05lld | pts:%05lld | ",seekpos,packet.pts); //lld I64d
			
			//decode video frame
			int ret = avcodec_decode_video2(codecCtx, decoded_frame, &got_picture, &packet);
			
			fprintf(stderr,"decode2 return val:%06d | ", ret);
			fprintf(stderr,"got_pic_ptr:%03d | ",got_picture);
			fprintf(stderr,"is keyframe:%d",is_keyframe);
			if(is_keyframe) is_keyframe = 0; //reset
			
			//got_picture = 1; //force display of frame
			if(got_picture){ //did we get a frame?

				//check first pixel value of first decoded YCbCr plane. Crashes when
				//got_picture is not set, proving that buffer is not being filled.
				fprintf(stderr," | data[0]:%d",*(decoded_frame->data[0]));
				
				frame_count++;
			}
			
			fprintf(stderr,"\n");
			
			/*
			If you let the loop iterate normally, you can see the PTS values
			returned for each frame. This works OK on both ffmpeg and ffmpeg-mt.
			If you then supply those exact same PTS values to av_seek_frame(),
			the decoder should return the exact same frame. This works as
			expected with normal ffmpeg, but not with ffmpeg-mt.
			*/
			//if(frame_count >= num_frames){	//check to see if we've reached the end of the stream
			if(frame_count >= 1){			//only process 1 frame when testing seeking
				break;
			}
		}
		
		//free packet populated by av_read_frame
		av_free_packet(&packet);
		
		//increment seek timestamp manually (doesn't seem to work in a loop)
		//seekpos+=1001;
	}
	
	//free AVFrame
	av_free(decoded_frame);

	//flush seek
	avcodec_flush_buffers(codecCtx);
	//avcodec_default_free_buffers(codecCtx);
		
	avcodec_close(codecCtx);
	av_close_input_file(formatCtx);
	
	return 0;
}
